---
title: "Something New is Brewing"
date: 2022-03-23
categories: [open-source, programming]
tags: [programming, homebrew, install tea]
draft: false
featured_image: "/Images/blog/something-new-is-brewing.jpg"
---
I created Homebrew nearly 13 years ago out of sheer necessity. I found the tooling available to developers at the time haphazard and difficult to tease into shape. After complaining about it at the pub one evening, I was met with an exasperated “Max, quit whining and do something about it”. So I did. (Thanks Jono).

From the start, I designed it to be delightful where other tools were not. I have always firmly believed when building anything, especially open source, that the tooling should get out of your way because you have much more important work to be doing.

I also intentionally designed it to _go viral_. I was lazy, you see, in the way all the best developers are. I needed you — _all of you_ — to help me build it. I did this so well that Homebrew became one of the biggest open source projects of all time.

Not long after starting, I quit my job to work on this project I felt I was put on the world to make. However… within a few months, the task was long from complete, and I found myself skint. I still remember going to the bank and asking what would happen if I overdrew the next day to pay for rent. They didn’t care much.

Thus began a steady rotation between jobs and open source. I wanted to work on open source full time, believing it would help the world become a better place. But that remained an elusive dream.

Eventually I burned out and quit maintaining Homebrew. I handed the reins to the community that had emerged from the trail I had laid out. I’m proud to say the project has done very well without me.

A few times a year someone would ask me if I would make a brew2. I always said no. What would be the point? brew is good, and I’m not going to reinvent the wheel.

## A Wild Timothy Appears

I met Timothy in Chicago in late 2013. We clicked and worked together on many projects. We were both passionate about what people can build in open environments that encourage and incentivize contribution. Eventually I moved away but the friendship stuck. Timothy caught the wave of crypto while I moved on to become a top mobile developer.

Every year or so he’d hit me up and talk to me about what was new in crypto, trying to get me to bite. I always declined. I admired the bitcoin white paper but I felt crypto was just money and, for me, money was the least interesting consequence of work.

## A Change of Heart

My partner and I were trying for a baby last year and in September, she showed me the positive test. We were overjoyed, but the next morning I woke up in a cold sweat. How was I going to provide for my family? _Open source was not paying the bills._

I went back over all my old ideas looking for a startup idea I could turn into a business. Eventually I came back to my ream of notes about “brew2”. Despite previously snubbing the idea of making another, I am an obsessive note taker and had been recording my thoughts as they came to me for years.

I moved on, at the time still convinced open source could not pay the bills.

Timothy phoned me to catch up and once more pointed me to what he was now calling “web3”.

I sniffed around the top of the rabbit hole.

While learning about web3 I bought and sold a few NFTs. The process was mostly uninteresting except for when I sold one and saw the automated, unavoidable 10% royalty enforced by a digital contract (with no need for a legal structure) that compensated the original creator for secondary sales_._ I felt the sting of inspiration.

_web3 enables indirect compensation._

I wondered if we could apply this concept to helping distribute value to open source.

I dove down the rabbit hole.

## The Nebraska Problem

It’s a well-known problem. Developers make fundamental improvements to the nature of the Internet. Their contributions are grabbed by the community with both hands and inserted into the tower:

![](https://miro.medium.com/v2/resize:fit:770/0*TTqemZPUKyxK8mgo)

[https://xkcd.com/2347/](https://xkcd.com/2347/)

Once a block is placed it’s rarely removed. The maintainers become charitable volunteers.

Solutions like sponsorship and bounties exist. Both unfortunately reward only the top of the tower, the visible packages. Beneath them lie tens of thousands of vital packages that most people don’t even know exist.

Recently Log4J [hit the news](https://www.cnet.com/news/privacy/log4j-software-bug-cisa-issues-emergency-directive-to-federal-agencies/) with a zero day that impacted most of the Internet. They fixed the bug and endured abuse while pointing out that they receive no funding from the myriad of unicorns that use them. They probably still don’t. Most of these hyper profitable corporations didn’t even know that Log4J was a vital brick in their tower of open source built by people they don’t even know and have never even paid.

CoreJS is another famous example, downloaded 30 million times a week. Their [README](https://github.com/zloirock/core-js/blob/master/README.md) is depressingly laced with _genuine need_ for funding to help them keep every Node.js application that exists afloat.

## Let’s Brew a Fresh Pot

Tools like Homebrew lie beneath all development tools, assisting developers to actually get development done. We know the graph of all open source, which means we’re uniquely placed to innovate in interesting and exciting ways. This is exactly what [tea](https://tea.xyz/) will do. We’re taking our knowledge of how to make development more efficient and throwing innovations nobody has ever really considered before. Package managers haven’t been sexy. Until now.

Most importantly, we’re moving the package registry on-chain (relax, we’ll use a low-energy proof of stake chain). This has numerous benefits due to the inherent benefits of blockchain technology:

-   Packages will be immutable (no more [left-pad incidents](https://www.theregister.com/2016/03/23/npm_left_pad_chaos/))
-   Packages will always be available (we’ll use decentralized storage)
-   Releases will be signed by the maintainers themselves (rather than a middleman you are told you can trust)
-   Tools can be built to fundamentally verify the integrity of your app’s open source constitution
-   Token can flow through the graph

Token flowing is where things get _really_ interesting, but first let’s set things straight: we’re not changing the nature of open source. It’s still free. web3 has enabled novel new ways to distribute value, and with our system people who care about the health of the open source ecosystem buy some token and stake it. Periodically, we reward this staking because it is securing our token network. We give a portion of these rewards to the staker and a portion to packages of their choice _along with all the dependencies of those packages_.

Note that no portion goes to _us_. We’re not like the other app stores.

## What is tea?

Founded by Max Howell, the creator of Homebrew and Timothy Lewis, famous web3 evangelist:

**tea** is a delightful product suite that you will adore.

**tea** is the home to a DAO that will ensure the open source maintainers that keep the Internet running are rewarded as they deserve.

**tea** is our revolution against a failing system.

Stay in touch: [https://tea.xyz](https://tea.xyz/) & [https://linktr.ee/teaxyz](https://linktr.ee/teaxyz)
